function [U,P]=linf_knownrotation(u,A,iter,upper);
%L-infinity structure and motion with known rotation
%[U,P] = LINF_KNOWNROTATION(u,A,iter,upper);
%
%u is a cell list of 2-by-m matrices of measured image coordinates.
% NB: Missing image points should be replaced by NaN.
%A is a cell list of 3-by-3 matrices of rotational part of the camera.
%iter (optional) is the number of bisection iterations. Default is 15.
%upper (optional) is the upper bound in pixels for initial interval. Default is 1.
%U is a 4-by-1 vector of 3D homogeneous point coordinates.
%P is a cell list of 3-by-4 camera matrices.
%
% (C) 2005 Fredrik Kahl.

if nargin<3 | isempty(iter),
    iter = 15;
end

if nargin<4 | isempty(upper),
    upper = 1;
end

low=0;
high=upper;
nbrimages=length(u);
if nbrimages<2,
    error('too few images');
end

nbrpoints=size(u{1},2);

%first 3d Point: [0,0,0,1]'
%first camera: [A0,[*,*,1]']


U=zeros(3,nbrpoints);
P=cell(1,nbrimages);

%parametrization: point coordinates, camera centres


feasible=0; %found a feasible solution?

%initializations...
res=zeros(1,nbrimages*nbrpoints);

gamma=high;
vars=3*(nbrpoints-1)+3*nbrimages-1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
At=sparse(zeros(0,vars));
c=sparse(zeros(0,1));
b=sparse(zeros(vars,1));
nbrcones=0;

clear K;
K.q=[];
for ii=1:nbrimages,
    Arot=A{ii};
    if ii==1,
        camindex=3*nbrpoints-2;
    else
        camindex=nbrpoints*3+ii*3-6;
    end
    uim=u{ii};
    for jj=1:nbrpoints,
        ptindex=3*jj-5;
        utmp=uim(:,jj);
        
        if ~isnan(utmp(1)),
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
            Atmp=sparse(zeros(3,vars));
            ctmp=sparse(zeros(3,1));
            if ii==1, %first centre [*,*,1]
                ctmp(1)=gamma; %p3
                ctmp(2:3)=-utmp; %p1,p2
            else
                Atmp(1,camindex+2)=-gamma; %p3
                Atmp(2:3,camindex+2)=utmp; %p1,p2
            end
            if jj>1, %first coordinate [0,0,0,1]
                Atmp(1,ptindex:ptindex+2)=-gamma*Arot(3,:);
            end
        
            %p14,p24
            Atmp(2,camindex)=-1;
            Atmp(3,camindex+1)=-1;

            if jj>1, %first coordinate [0,0,0,1]
                Atmp(2:3,ptindex:ptindex+2)=utmp*Arot(3,:)-Arot(1:2,:);
            end
            At=[At;Atmp];
            c=[c;ctmp];

            K.q=[K.q,3];
            nbrcones=nbrcones+1;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
        end %if ~isnan(utmp(1)),
    end %jj=1:nbrpoints
end %ii=1:nbrimages

pars=[];
pars.fid=0;

qq=0;
while qq<iter & high-low>1e-8,
    qq=qq+1;
    [x,y,info]=sedumi(At,b,c,K,pars);
    gammaold = gamma;

    if info.dinf==0,
        %evaluate estimated point
        U(4:end)=[y(1:3*nbrpoints-3)];
        
        cnt=0;
        for ii=1:nbrimages,
            if ii==1,
                camindex=3*(nbrpoints-1)+[1:2];
                Ptmp=[A{1},[y(camindex);1]];
            else
                camindex=3*nbrpoints+3*ii-6+[0:2];
                Ptmp=[A{ii},y(camindex)];
            end
            P{ii}=Ptmp;
            uim=u{ii};
            tmp=pflat(Ptmp*[U;ones(1,nbrpoints)]);
            res(cnt+[1:nbrpoints])=sum((uim-tmp(1:2,:)).^2);
            cnt=cnt+nbrpoints;
         end
         high=sqrt(max(res));
         feasible=1;
    else
        %'infeasible'
        low=gamma;
        if feasible==0,
            high=high*2;
        end
    end
    
    gamma=(high+low)/2;
    
    if qq<iter, %more iterations
        %apply new gamma to c and At
        gfactor=gamma/gammaold; 
        for ii=1:nbrcones,
            index=3*ii-2;
            c(index)=c(index)*gfactor;
            At(index,:)=At(index,:)*gfactor;
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%
end %gamma iteration

if feasible,
    U(4,:)=1;
else
    %no feasible solution
    U=[];
    P={};
end

