function P=linf_resection(u,U,iter,upper);
%L-infinity resection from n points
%P = LINF_RESECTION(u,U,iter,upper);
%
%u is a 2-by-n matrix of measured image coordinates.
%U is a 4-by-n matrix of 3D homogeneous point coordinates.
%iter (optional) is the number of bisection iterations. Default is 15.
%upper (optional) is the upper bound in pixels for initial interval. Default is 1.
%P is a 3-by-4 camera matrix.
%
% (C) 2005 Fredrik Kahl.

if nargin<3 | isempty(iter),
    iter=15;
end
if nargin<4 | isempty(upper),
    upper=1;
end

%translate first point to origin
trans=pflat(U(:,1));
T=eye(4);T(1:3,4)=-trans(1:3);
Ut=pflat(T*U);

low=0;
high=upper;

%parametrization: P=[p1,p2,p3,p4;p5,p6,p7,p8;p9,p10,p11,1]

if size(u,1)~=2,
    error('wrong size on image coordinate matrix');
end
if size(U,1)~=4,
    error('wrong size on 3D coordinate matrix');
end

%number of points
nbrpoints=min(size(u,2),size(U,2));
if nbrpoints<6,
    error('too few points');
end
feasible=0; %found a feasible solution?
gamma=high;
vars=11; %number of variables

%sedumi matrices
At=sparse(zeros(0,vars));
c=sparse(zeros(0,1));
b=sparse(zeros(vars,1));

clear K;
K.q=[];
for ii=1:nbrpoints,
    Atmp=sparse(zeros(3,vars));
    ctmp=sparse(zeros(3,1));
    
    Utmp=Ut(:,ii); %current points
    utmp=u(:,ii);
    
    %radius of cone constraint
    ctmp(1)=gamma*Utmp(4);
    Atmp(1,9:11)=-gamma*Utmp(1:3)';

    %coefficients of cone constraint
    ctmp(2:3)=-utmp*Utmp(4);
    Atmp(2,1:4)=-Utmp';
    Atmp(3,5:8)=-Utmp';
    Atmp(2:3,9:11)=utmp*Utmp(1:3)';

    %store in Sedumi matrices
    At=[At;Atmp];
    c=[c;ctmp];
    K.q=[K.q,3];
end

pars=[];
pars.fid=0;

qq=0;
while qq<iter & high-low>1e-8,
    qq=qq+1;
    
    [x,y,info]=sedumi(At,b,c,K,pars);
    gammaold = gamma;

    infeasible=1;
    if info.dinf==0,
        %evaluate estimated point
        Ptmp=ones(4,3);
        Ptmp(1:11)=real(y);
        Ptmp=Ptmp';
        
        tmp=pflat(Ptmp*Ut);
        res=sum((u(1:2,:)-tmp(1:2,:)).^2);
        maxres=sqrt(max(res));
        
        if maxres<=gamma,
            high=maxres;
            P=Ptmp;
            feasible=1;
            infeasible=0;
        end
    end
    if infeasible,
        %'infeasible' -> make interval larger
        low=gamma;
        if feasible==0,
            high=high*2;
        end
    end
    
    gamma=(high+low)/2; %bisection
    
    if qq<iter, %more iterations
        %apply new gamma to c and At
        gfactor=gamma/gammaold;
        index=1:3:3*nbrpoints;
        c(index)=c(index)*gfactor;
        At(index,:)=At(index,:)*gfactor;
    end
end %gamma iteration


if feasible==0,
    P=[];
else
    P=P*T; %transform to original coordinate system
end

