function U=linf_triangulation(u,P,iter,upper);
%L-infinity triangulation from m views
%U = LINF_TRIANGULATION(u,P,iter,upper);
%
%u is a 2-by-m matrix of measured image coordinates.
%P is a cell list of 3-by-4 camera matrices.
%iter (optional) is the number of bisection iterations. Default is 15.
%upper (optional) is the upper bound in pixels for initial interval. Default is 1.
%U is a 4-by-1 vector of 3D homogeneous point coordinates.
%
%Note: the sign of camera matrices should be set so that points are in front of cameras.
%
% (C) 2005 Fredrik Kahl.

if nargin<3 | isempty(iter),
    iter = 15;
end

if nargin<4 | isempty(upper),
    upper = 1;
end

low=0;
high=upper;
nbrimages=size(u,2);
if nbrimages<2,
    error('too few images');
end
U=[];
feasible=0; %found a feasible solution?
gamma=high;

%Sedumi matrices
At=sparse(zeros(0,3));
c=sparse(zeros(0,1));
b=sparse(0);
clear K;
K.q=[];
for ii=1:nbrimages,
    Atmp=sparse(zeros(3,3));
    ctmp=sparse(zeros(3,1));
    
    Pcam=P{ii};
    
    %radius of cone constraint
    ctmp(1)=gamma*Pcam(3,4);
    Atmp(1,1:3)=-gamma*Pcam(3,1:3);
    
    %coefficients of cone constraint
    ctmp(2:3)=Pcam(1:2,4)-u(1:2,ii)*Pcam(3,4);
    Atmp(2:3,1:3)=-Pcam(1:2,1:3)+u(1:2,ii)*Pcam(3,1:3);

    %store in Sedumi matrices
    At=[At;Atmp];
    c=[c;ctmp];
    K.q=[K.q,3];
end

pars=[];
pars.fid=0;

qq=0;
while qq<iter & high-low>1e-8,
    qq=qq+1;
    [x,y,info]=sedumi(At,b,c,K,pars);
    gammaold = gamma;
    infeasible=1;

    if info.dinf==0 & info.numerr==0,
        %evaluate estimated point
        res=zeros(nbrimages,1);
        for ii=1:nbrimages,
            tmp=pflat(P{ii}*[y;1]);
            res(ii)=sum((u(:,ii)-tmp(1:2)).^2);
        end
        maxres=sqrt(max(res));
     
        if maxres<=gamma,
            high=maxres;
            feasible=1;
            infeasible=0;
            U=[y;1];
        end
    end
    if infeasible,
        %'infeasible' -> make interval larger
        low=gamma;
        if feasible==0,
            high=high*2;
        end
    end
    
    gamma=(high+low)/2; %bisection
    
    if qq<iter, %more iterations
        %apply new gamma to c and At
        gfactor=gamma/gammaold;
        index=1:3:3*nbrimages;
        c(index)=c(index)*gfactor;
        At(index,:)=At(index,:)*gfactor;
    end
end %gamma iteration
